@@ -2,6 +2,7 @@ pub mod os_input_output;
 pub mod panes;
 pub mod tab;
 
+mod logging_pipe;
 mod pty;
 mod route;
 mod screen;
@@ -9,11 +10,14 @@ mod thread_bus;
 mod ui;
 mod wasm_vm;
 
+use log::info;
+use std::{
+    path::PathBuf,
+    sync::{Arc, Mutex, RwLock},
+    thread,
+};
 use zellij_utils::zellij_tile;
 
-use std::path::PathBuf;
-use std::sync::{Arc, Mutex, RwLock};
-use std::thread;
 use wasmer::Store;
 use zellij_tile::data::{Event, Palette, PluginCapabilities};
 
@@ -22,7 +26,6 @@ use crate::{
     pty::{pty_thread_main, Pty, PtyInstruction},
     screen::{screen_thread_main, ScreenInstruction},
     thread_bus::{Bus, ThreadSenders},
-    ui::layout::Layout,
     wasm_vm::{wasm_thread_main, PluginInstruction},
 };
 use route::route_thread_main;
@@ -30,7 +33,12 @@ use zellij_utils::{
     channels::{self, ChannelWithContext, SenderWithContext},
     cli::CliArgs,
     errors::{ContextType, ErrorInstruction, ServerContext},
-    input::{get_mode_info, options::Options},
+    input::{
+        command::{RunCommand, TerminalAction},
+        get_mode_info,
+        layout::LayoutFromYaml,
+        options::Options,
+    },
     ipc::{ClientAttributes, ClientToServerMsg, ExitReason, ServerToClientMsg},
     setup::get_default_data_dir,
 };
@@ -38,7 +46,7 @@ use zellij_utils::{
 /// Instructions related to server-side application
 #[derive(Debug, Clone)]
 pub(crate) enum ServerInstruction {
-    NewClient(ClientAttributes, Box<CliArgs>, Box<Options>),
+    NewClient(ClientAttributes, Box<CliArgs>, Box<Options>, LayoutFromYaml),
     Render(Option<String>),
     UnblockInputThread,
     ClientExit,
@@ -50,8 +58,8 @@ pub(crate) enum ServerInstruction {
 impl From<ClientToServerMsg> for ServerInstruction {
     fn from(instruction: ClientToServerMsg) -> Self {
         match instruction {
-            ClientToServerMsg::NewClient(attrs, opts, options) => {
-                ServerInstruction::NewClient(attrs, opts, options)
+            ClientToServerMsg::NewClient(attrs, opts, options, layout) => {
+                ServerInstruction::NewClient(attrs, opts, options, layout)
             }
             ClientToServerMsg::AttachClient(attrs, force, options) => {
                 ServerInstruction::AttachClient(attrs, force, options)
@@ -85,6 +93,7 @@ pub(crate) struct SessionMetaData {
     pub senders: ThreadSenders,
     pub capabilities: PluginCapabilities,
     pub palette: Palette,
+    pub default_shell: Option<TerminalAction>,
     screen_thread: Option<thread::JoinHandle<()>>,
     pty_thread: Option<thread::JoinHandle<()>>,
     wasm_thread: Option<thread::JoinHandle<()>>,
@@ -109,12 +118,10 @@ pub(crate) enum SessionState {
 }
 
 pub fn start_server(os_input: Box<dyn ServerOsApi>, socket_path: PathBuf) {
-    #[cfg(not(any(feature = "test", test)))]
+    info!("Starting Zellij server!");
     daemonize::Daemonize::new()
         .working_directory(std::env::current_dir().unwrap())
         .umask(0o077)
-        // FIXME: My cherished `dbg!` was broken, so this is a hack to bring it back
-        //.stderr(std::fs::File::create("dbg.log").unwrap())
         .start()
         .expect("could not daemonize the server process");
 
@@ -125,7 +132,6 @@ pub fn start_server(os_input: Box<dyn ServerOsApi>, socket_path: PathBuf) {
     let session_data: Arc<RwLock<Option<SessionMetaData>>> = Arc::new(RwLock::new(None));
     let session_state = Arc::new(RwLock::new(SessionState::Uninitialized));
 
-    #[cfg(not(any(feature = "test", test)))]
     std::panic::set_hook({
         use zellij_utils::errors::handle_panic;
         let to_server = to_server.clone();
@@ -136,21 +142,6 @@ pub fn start_server(os_input: Box<dyn ServerOsApi>, socket_path: PathBuf) {
 
     let thread_handles = Arc::new(Mutex::new(Vec::new()));
 
-    #[cfg(any(feature = "test", test))]
-    thread_handles.lock().unwrap().push(
-        thread::Builder::new()
-            .name("server_router".to_string())
-            .spawn({
-                let session_data = session_data.clone();
-                let os_input = os_input.clone();
-                let to_server = to_server.clone();
-                let session_state = session_state.clone();
-
-                move || route_thread_main(session_data, session_state, os_input, to_server)
-            })
-            .unwrap(),
-    );
-    #[cfg(not(any(feature = "test", test)))]
     let _ = thread::Builder::new()
         .name("server_listener".to_string())
         .spawn({
@@ -208,25 +199,44 @@ pub fn start_server(os_input: Box<dyn ServerOsApi>, socket_path: PathBuf) {
         let (instruction, mut err_ctx) = server_receiver.recv().unwrap();
         err_ctx.add_call(ContextType::IPCServer((&instruction).into()));
         match instruction {
-            ServerInstruction::NewClient(client_attributes, opts, config_options) => {
+            ServerInstruction::NewClient(client_attributes, opts, config_options, layout) => {
                 let session = init_session(
                     os_input.clone(),
                     opts,
-                    config_options,
+                    config_options.clone(),
                     to_server.clone(),
                     client_attributes,
                     session_state.clone(),
+                    layout.clone(),
                 );
                 *session_data.write().unwrap() = Some(session);
                 *session_state.write().unwrap() = SessionState::Attached;
-                session_data
-                    .read()
-                    .unwrap()
-                    .as_ref()
-                    .unwrap()
-                    .senders
-                    .send_to_pty(PtyInstruction::NewTab)
-                    .unwrap();
+
+                let default_shell = config_options.default_shell.map(|shell| {
+                    TerminalAction::RunCommand(RunCommand {
+                        command: shell,
+                        ..Default::default()
+                    })
+                });
+
+                let spawn_tabs = |tab_layout| {
+                    session_data
+                        .read()
+                        .unwrap()
+                        .as_ref()
+                        .unwrap()
+                        .senders
+                        .send_to_pty(PtyInstruction::NewTab(default_shell.clone(), tab_layout))
+                        .unwrap()
+                };
+
+                if !&layout.tabs.is_empty() {
+                    for tab_layout in layout.tabs {
+                        spawn_tabs(Some(tab_layout.clone()));
+                    }
+                } else {
+                    spawn_tabs(None);
+                }
             }
             ServerInstruction::AttachClient(attrs, _, options) => {
                 *session_state.write().unwrap() = SessionState::Attached;
@@ -234,7 +244,7 @@ pub fn start_server(os_input: Box<dyn ServerOsApi>, socket_path: PathBuf) {
                 let session_data = rlock.as_ref().unwrap();
                 session_data
                     .senders
-                    .send_to_screen(ScreenInstruction::TerminalResize(attrs.position_and_size))
+                    .send_to_screen(ScreenInstruction::TerminalResize(attrs.size))
                     .unwrap();
                 let default_mode = options.default_mode.unwrap_or_default();
                 let mode_info =
@@ -292,7 +302,6 @@ pub fn start_server(os_input: Box<dyn ServerOsApi>, socket_path: PathBuf) {
         .unwrap()
         .drain(..)
         .for_each(|h| drop(h.join()));
-    #[cfg(not(any(feature = "test", test)))]
     drop(std::fs::remove_file(&socket_path));
 }
 
@@ -303,6 +312,7 @@ fn init_session(
     to_server: SenderWithContext<ServerInstruction>,
     client_attributes: ClientAttributes,
     session_state: Arc<RwLock<SessionState>>,
+    layout: LayoutFromYaml,
 ) -> SessionMetaData {
     let (to_screen, screen_receiver): ChannelWithContext<ScreenInstruction> = channels::unbounded();
     let to_screen = SenderWithContext::new(to_screen);
@@ -323,18 +333,12 @@ fn init_session(
         arrow_fonts: config_options.simplified_ui,
     };
 
-    // Don't use default layouts in tests, but do everywhere else
-    #[cfg(not(any(feature = "test", test)))]
-    let default_layout = Some(PathBuf::from("default"));
-    #[cfg(any(feature = "test", test))]
-    let default_layout = None;
-    let layout_path = opts.layout_path;
-    let maybe_layout = opts
-        .layout
-        .as_ref()
-        .map(|p| Layout::from_dir(&p, &data_dir))
-        .or_else(|| layout_path.map(|p| Layout::new(&p)))
-        .or_else(|| default_layout.map(|p| Layout::from_dir(&p, &data_dir)));
+    let default_shell = config_options.default_shell.clone().map(|command| {
+        TerminalAction::RunCommand(RunCommand {
+            command,
+            ..Default::default()
+        })
+    });
 
     let pty_thread = thread::Builder::new()
         .name("pty".to_string())
@@ -351,7 +355,7 @@ fn init_session(
                 opts.debug,
             );
 
-            move || pty_thread_main(pty, maybe_layout)
+            move || pty_thread_main(pty, layout)
         })
         .unwrap();
 
@@ -387,7 +391,7 @@ fn init_session(
                 vec![plugin_receiver],
                 Some(&to_screen),
                 Some(&to_pty),
-                None,
+                Some(&to_plugin),
                 None,
                 None,
             );
@@ -402,8 +406,10 @@ fn init_session(
             to_pty: Some(to_pty),
             to_plugin: Some(to_plugin),
             to_server: None,
+            should_silently_fail: false,
         },
         capabilities,
+        default_shell,
         palette: client_attributes.palette,
         screen_thread: Some(screen_thread),
         pty_thread: Some(pty_thread),
