@@ -8,9 +8,8 @@ fn read_fixture(fixture_name: &str) -> Vec<u8> {
     path_to_file.push("tests");
     path_to_file.push("fixtures");
     path_to_file.push(fixture_name);
-    let content = std::fs::read(path_to_file)
-        .unwrap_or_else(|_| panic!("could not read fixture {:?}", &fixture_name));
-    content
+    std::fs::read(path_to_file)
+        .unwrap_or_else(|_| panic!("could not read fixture {:?}", &fixture_name))
 }
 
 #[test]
